==> lkrg-0.6.tar.gz: lkrg-0.6/src/modules/exploit_detection/syscalls/pCFI/p_schedule/p_schedule.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept '__schedule' function
 *
 * Notes:
 *  - Enforce Exploit Detection CFI
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 20.XI.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../../p_lkrg_main.h"


char p_pcfi_schedule_kretprobe_state = 0x0;

static struct kretprobe p_pcfi_schedule_kretprobe = {
    .kp.symbol_name = "schedule",
    .handler = p_pcfi_schedule_ret,
    .entry_handler = p_pcfi_schedule_entry_smep,
    .data_size = sizeof(struct p_pcfi_schedule_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

static inline void p_write_cr4(unsigned long p_arg) {

#if LINUX_VERSION_CODE < KERNEL_VERSION(4,0,0)
   write_cr4(p_arg);
#else
   __write_cr4(p_arg);
#endif

}

static inline unsigned long p_read_cr4(void) {

#if LINUX_VERSION_CODE < KERNEL_VERSION(4,0,0)
   return read_cr4();
#else
   return __read_cr4();
#endif

}

int p_pcfi_schedule_entry_nosmep(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_cr0;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_schedule_entry>\n");

//   p_ed_enforce_validation();

   spin_lock(&p_rb_ed_pids_lock);
   p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current));
   if (p_ed_enforce_pcfi(current, p_regs, (p_tmp) ? p_tmp->p_ed_task.p_stack : NULL)) {
      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Stack pointer corruption (ROP?) - pCFI violation: process[%s | %d] !!!\n",
             current->comm,task_pid_nr(current));
      // kill this process!
      p_ed_kill_task_by_task(current);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_cr0 = read_cr0();
   if (!(p_cr0 & X86_CR0_WP)) {
      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Write Protection bit was disabled! Enforcing WP now!\n");
      p_cr0 |= X86_CR0_WP;
      write_cr0(p_cr0);
      wbinvd();
   }

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_schedule_entry>\n");

   return 0x0;
}

int p_pcfi_schedule_entry_smep(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_cr0;
   unsigned long p_cr4;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_schedule_entry>\n");

//   p_ed_enforce_validation();

   spin_lock(&p_rb_ed_pids_lock);
   p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current));
   if (p_ed_enforce_pcfi(current, p_regs, (p_tmp) ? p_tmp->p_ed_task.p_stack : NULL)) {
      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Stack pointer corruption (ROP?) - pCFI violation: process[%s | %d] !!!\n",
             current->comm,task_pid_nr(current));
      // kill this process!
      p_ed_kill_task_by_task(current);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_cr4 = p_read_cr4();
   if (!(p_cr4 & X86_CR4_SMEP)) {
      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> SMEP was disabled! Enforcing SMEP now!\n");
//      cr4_set_bits(X86_CR4_SMEP);
      p_cr4 |= X86_CR4_SMEP;
      p_write_cr4(p_cr4);
   }

   p_cr0 = read_cr0();
   if (!(p_cr0 & X86_CR0_WP)) {
      p_print_log(P_LKRG_CRIT,
             "<Exploit Detection> Write Protection bit was disabled! Enforcing WP now!\n");
      p_cr0 |= X86_CR0_WP;
      write_cr0(p_cr0);
      wbinvd();
   }

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_schedule_entry>\n");

   return 0x0;
}


int p_pcfi_schedule_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_schedule_ret>\n");

//   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_schedule_ret>\n");

   return 0x0;
}


int p_install_pcfi_schedule_hook(unsigned long p_global_SMEP) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_pcfi_schedule_hook>\n");

   if (unlikely(!p_global_SMEP)) {
      p_pcfi_schedule_kretprobe.entry_handler = p_pcfi_schedule_entry_nosmep;
   }

   if ( (p_ret = register_kretprobe(&p_pcfi_schedule_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_pcfi_schedule_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_pcfi_schedule_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_pcfi_schedule_kretprobe.kp.symbol_name,
               p_pcfi_schedule_kretprobe.kp.addr);
   p_pcfi_schedule_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_pcfi_schedule_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_pcfi_schedule_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_pcfi_schedule_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_pcfi_schedule_hook>\n");

   if (!p_pcfi_schedule_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_pcfi_schedule_kretprobe.kp.symbol_name,
                  p_pcfi_schedule_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_pcfi_schedule_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_pcfi_schedule_kretprobe.kp.symbol_name,
                  p_pcfi_schedule_kretprobe.kp.addr,
                  p_pcfi_schedule_kretprobe.nmissed);
      p_pcfi_schedule_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_pcfi_schedule_hook>\n");
}

==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/exploit_detection/syscalls/pCFI/p_schedule/p_schedule.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept '__schedule' function
 *
 * Notes:
 *  - Enforce Exploit Detection CFI
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 20.XI.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../../p_lkrg_main.h"


char p_pcfi_schedule_kretprobe_state = 0x0;

static struct kretprobe p_pcfi_schedule_kretprobe = {
    .kp.symbol_name = "schedule",
    .handler = p_pcfi_schedule_ret,
    .entry_handler = p_pcfi_schedule_entry,
    .data_size = sizeof(struct p_pcfi_schedule_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_pcfi_schedule_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_schedule_entry>\n");

   p_ed_pcfi_cpu(1);

//   p_ed_enforce_validation();

   if (p_is_ed_task(current)) {
      p_tasks_read_lock(&p_flags);
      if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
         if (p_ed_enforce_pcfi(current, p_tmp, p_regs)) {
            p_print_log(P_LKRG_CRIT,
                   "<Exploit Detection> Stack pointer corruption (ROP?) - pCFI violation: process[%s | %d] !!!\n",
                   current->comm,task_pid_nr(current));
            // kill this process!
            p_pcfi_kill_task_by_task(current);
         }
      }
      p_tasks_read_unlock(&p_flags);
   }

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_schedule_entry>\n");

   return 0x0;
}


int p_pcfi_schedule_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_pcfi_schedule_ret>\n");

//   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_pcfi_schedule_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(pcfi_schedule)
